@@ -13,6 +13,7 @@ from Crypto.Hash import SHA256
 from pbkdf2 import PBKDF2
 # ##################################################################################################
 from n0struct import *
+DEBUG_MODE = False
 # ##################################################################################################
 class n0Vault(dict):
     _encrypted = False
@@ -176,8 +177,15 @@ class n0Vault(dict):
 
         Load 'vault_file_name' and decrypt it if it was encrypted.
         """
-        def read_buffer(len_to_read = None, name = None):
-            return in_file.read(len_to_read)
+        # ******************************************************************************************
+        def read_buffer(len_to_read: int = -1, name_of_buffer: str = "") -> bytes:
+            if not len_to_read:
+                len_to_read = -1
+            __buffer = in_file.read(len_to_read)
+            if DEBUG_MODE:
+                n0debug_calc(__buffer, name_of_buffer)
+            return __buffer
+        # ******************************************************************************************
 
         self.vault_file_name = vault_file_name
         if os.path.exists(self.vault_file_name):
@@ -186,13 +194,13 @@ class n0Vault(dict):
                 if sign == b'{':
                     # Not-crypted storage
                     self.__vault_file_is_encrypted = False
-                    self._vault = n0dict((sign + read_buffer()).decode("utf-8"))
+                    self._vault = n0dict((sign + read_buffer(-1, "the whole json")).decode("utf-8"))
                 else:
                     # Encrypted storage
                     self.__vault_file_is_encrypted = True
                     sign += read_buffer(7, "sign")
                     if sign.decode("utf-8") != self.__sign:
-                        raise Exception(f"File '{vault_file_name}' is not n0Vault storage")
+                        raise TypeError(f"File '{vault_file_name}' is not n0Vault storage")
                     # 4 bytes/32-bits flag
                     self.__flags = int.from_bytes(read_buffer(4, "flags"), 'little')
                     cipher_iv = read_buffer(16, "cipher_iv")
@@ -203,33 +211,33 @@ class n0Vault(dict):
                         cipher = AES.new(self.__key, AES.MODE_GCM, cipher_iv)
                     elif self.is_bit_set(0, 0b11) == 0b01:
                         if not self.__password:
-                            raise Exception(f"Password for loading is required")
+                            raise TypeError("Password for loading is required")
                         cipher = AES.new(
                             PBKDF2(self.__password, self.__key[:16]).read(32),    # 256-bit key
                             AES.MODE_GCM,
                             cipher_iv
                         )
                     else:
-                        raise Exception(f"Unknown format of encryption for n0Vault storage")
+                        raise TypeError("Unknown format of encryption for n0Vault storage")
                     # ******************************************************************************
                     try:
-                        buffer = Crypto.Util.Padding.unpad(
+                        decrypted_buffer = Crypto.Util.Padding.unpad(
                                     cipher.decrypt(
-                                            read_buffer(None, "encrypted buffer")
+                                            read_buffer(-1, "encrypted buffer")
                                         ),
                                     AES.block_size
                         )
                     except:
-                        raise Exception(f"Incorrect password for n0Vault storage")
+                        raise PermissionError("Incorrect password for n0Vault storage")
                     # ******************************************************************************
                     # ******************************************************************************
-                    calculated_control_sum = SHA256.new(data=cipher_iv + buffer).digest()
+                    calculated_control_sum = SHA256.new(data=cipher_iv + decrypted_buffer).digest()
                     if control_sum != calculated_control_sum:
-                        raise Exception(f"Incorrect control sum of n0Vault storage")
-                    self._vault = n0dict(buffer.decode("utf-8"))
+                        raise TypeError("Incorrect control sum of n0Vault storage")
+                    self._vault = n0dict(decrypted_buffer.decode("utf-8"))
 
                 if self._vault.get("__sign") != self.__sign:
-                    raise Exception(f"Incorrect format of n0Vault storage")
+                    raise TypeError("Incorrect format of n0Vault storage")
         else:
             self._vault = n0dict({"__sign": self.__sign})
         return self._vault
@@ -242,17 +250,22 @@ class n0Vault(dict):
         if 3rd bit in self.__flags is already set previously, then Exception will be raised -- saving is forbidden.
         """
         if self.is_bit_set(2, 0b1):
-            raise Exception(f"Saving of such n0Vault storage is forbidden")
+            raise PermissionError("Saving of such n0Vault storage is forbidden")
         # ******************************************************************************************
         def write_buffer(buffer: typing.Union[str, int, bytes], name_of_buffer: str):
-            n0debug_calc(buffer, name_of_buffer)
-            if isinstance(buffer, str):
-                buffer = buffer.encode("utf-8")             # str -> bytes
-            elif isinstance(buffer, int):
-                buffer = buffer.to_bytes(4, 'little')       # int32 -> bytes
-            elif not isinstance(buffer, bytes):
-                raise(Exception(f"Expected type str or int for '{buffer}', but got {type(buffer)}"))
-            out_file.write(buffer)
+            if DEBUG_MODE:
+                n0debug_calc(buffer, name_of_buffer)
+                
+            if isinstance(buffer, bytes):
+                __buffer = buffer                           # request of sonarcloud.io
+            else:
+                if isinstance(buffer, str):
+                    __buffer = buffer.encode("utf-8")       # str -> bytes
+                elif isinstance(buffer, int):
+                    __buffer = buffer.to_bytes(4, 'little') # int32 -> bytes
+                else:
+                    raise TypeError(f"Expected type str or int for '{buffer}', but got {type(buffer)}")
+            out_file.write(__buffer)
         # ******************************************************************************************
         with open(new_vault_file_name or self.vault_file_name, "wb") as out_file:
             if self._encrypted is None:
@@ -271,20 +284,20 @@ class n0Vault(dict):
                     self.__flags = self.set_bits(bytes_array = self.__flags, bits_value = 0b1, bits_len = 1, bits_offset = 2)
                 write_buffer(self.__flags, "flags")
                 write_buffer(cipher.iv,    "cipher.iv")
-                buffer = n0pretty(self._vault, show_type=False, __indent_size = 0).encode("utf-8") # str -> bytes
-                write_buffer(SHA256.new(data=cipher.iv + buffer).digest(), "control_sum")
+                packed_json = n0pretty(self._vault, show_type=False, __indent_size = 0).encode("utf-8") # str -> bytes
+                write_buffer(SHA256.new(data=cipher.iv + packed_json).digest(), "control_sum")
                 write_buffer(
                             cipher.encrypt(
                                     Crypto.Util.Padding.pad(
-                                        buffer,
+                                        packed_json,
                                         AES.block_size
                                     )
                             ),
                             "encrypted buffer"
                 )
             else:
-                buffer = self.show().replace('\n', "\r\n").encode("utf-8")  # str -> bytes
-                write_buffer(buffer, "notcrypted buffer")
+                unpacked_json = self.show().replace('\n', "\r\n").encode("utf-8")  # str -> bytes
+                write_buffer(unpacked_json, "notcrypted buffer")
     # **********************************************************************************************
     def __getitem__(self, xpath):
         """
